library reactive_property;

import 'dart:async';
import 'package:meta/meta.dart';
import 'package:rxdart/rxdart.dart';

/// An abstraction layer for RxDart Subjects.
/// Reactive example:
/// ```
/// class RxCounter extends Reactive<int> {
///   RxCounter() : super(0);
///
///   void increment() => value++;
///   void decrement() => value--;
/// }
///```
mixin ReactiveMixin<T> {
  // The internel stream controller
  Subject<T> _subject;

  /// Return whether the controller has already been closed or not.
  /// Once closed, it is not allowed to change the value.
  ///
  /// If add() or the set value is used after the stream is closed an exception will be thrown.
  bool isClosed = false;

  T _value;

  /// Returns the latest emited value by the Reactive.
  T get value => _value;

  /// Sends a new data [value].
  /// Is equivalent to add()
  @protected
  set value(T value) => add(value);

  /// Sends a new data [value].
  @protected
  void add(T value) async {
    _value = value;
    _subject.add(value);
  }

  void addError(Object error, [StackTrace stackTrace]) {
    _subject.addError(error, stackTrace);
  }

  StreamSubscription<T> listen(
    void Function(T) onData, {
    Function onError,
    void Function() onDone,
    bool cancelOnError,
  }) {
    return _subject.listen(
      onData,
      onError: onError,
      onDone: onDone,
      cancelOnError: cancelOnError,
    );
  }

  bool get isBroadcast => true;

  /// Closes the sink.
  /// The [add] method and use the set [value] must not be called after this method.
  /// Calling this method more than once is allowed, but does nothing.
  Future<void> close() async {
    isClosed = true;
    if (isClosed) return;
    await _subject.close();
  }
}

class ReactiveController<T> extends Stream<T>
    with ReactiveMixin<T>
    implements EventSink<T> {
  ReactiveController(T initalValue, Subject<T> subject) {
    _subject = subject;
    _value = initalValue;
  }
}

abstract class Reactive<T> extends ReactiveController<T> {
  Reactive(T initalValue)
      : super(initalValue, BehaviorSubject.seeded(initalValue));

  void change(T Function(T value) changer) {
    super.add(changer(value));
  }
}

class ReactiveProperty<T> extends Reactive<T> {
  ReactiveProperty(T initalValue) : super(initalValue);
  @override
  set value(T value) => super.value = value;

  @override
  add(T value) => super.add;

  static ReactiveProperty<T> replay<T>(T initalValue) {
    return ReplayReactiveProperty(initalValue);
  }

  static ReactiveProperty<T> publish<T>(T initalValue) {
    return PublishReactiveProperty(initalValue);
  }
}

abstract class ReplayReactive<T> extends ReactiveController<T> {
  ReplayReactive(T initalValue) : super(initalValue, ReplaySubject());
}

class PublishReactive<T> extends ReactiveController<T> {
  PublishReactive(T initalValue) : super(initalValue, PublishSubject());
}

class ReplayReactiveProperty<T> extends ReactiveProperty<T> {
  ReplayReactiveProperty(T initalValue) : super(initalValue);
}

class PublishReactiveProperty<T> extends ReactiveProperty<T> {
  PublishReactiveProperty(T initalValue) : super(initalValue);
}
