import 'package:reactive_property/reactive_property.dart';
import 'package:rxdart/rxdart.dart';
import 'package:test/test.dart';

abstract class Bloc {
  void dispose();
}

// Using only RxDart
class CounterBloc extends Bloc {
  final counterController = BehaviorSubject.seeded(0);

  Stream<int> get stream => counterController.stream;

  void increment() {
    counterController.add(counterController.value + 1);
  }

  @override
  void dispose() {
    counterController.close();
  }
}

// Using RxState
class ReactiveCounter extends Reactive<int> {
  ReactiveCounter() : super(0);
  void increment() => value++;
}

// Or
//final counter = Reactive(0);
//counter.value++;

// Then you can use its reactive value.
void main() {
  final counter = ReactiveCounter();
  final counter2 = ReactiveProperty(0);

  //Emits a new value easily.
  counter.increment();
  counter2.value++;
  // Listen the changes easily
  counter.listen(print);
  counter2.listen(print);

  // Retrieve the current value.
  print(counter2.value);
  print(counter.value);

  //Then close it.
  counter2.close();
  counter.close();
}

// Simple as ValueNotifier and powerful as RxDart.

class RxCounter {
  final counter = ReactiveProperty(0);
  final counter2 = ReactiveProperty(0);

  void increment() => counter.value++;
}

void mainx() {
  ReactiveProperty reactiveCounter;
  Reactive<int> rxCounter;

  setUp(() {
    reactiveCounter = ReactiveProperty(0);
    rxCounter = ReactiveProperty(10);
  });

  group('state', () {
    test('shoud start with initial state', () {
      expect(reactiveCounter.value, 10);
    });

    test('shoud be broadcast', () {
      expect(reactiveCounter.isBroadcast, true);
    });

    test('shoud return the current state after add a new state', () {
      reactiveCounter.add(20);
      expect(reactiveCounter.value, 20);
    });

    group('reactive property', () {
      test('shoud start with initial state', () {
        expect(rxCounter.value, 10);
      });
    });
  });

  group('emit new state', () {
    group('add', () {
      test('shoud emit corret state', () async {
        expectLater(reactiveCounter, emitsInOrder([10, 20]));

        reactiveCounter.add(20);
      });

      test('shoud emit corret state ir order', () async {
        expectLater(reactiveCounter, emitsInOrder([10, 20, 30]));

        reactiveCounter.add(20);
        await Future.delayed(Duration.zero);
        reactiveCounter.add(30);
      });
    });

    group('state set', () {
      test('shoud emit corret state', () async {
        expectLater(reactiveCounter, emitsInOrder([10, 20]));

        reactiveCounter.add(20);
      });

      test('shoud emit corret state ir order', () async {
        expectLater(reactiveCounter, emitsInOrder([10, 20, 30]));

        reactiveCounter.value = 20;
        await Future.delayed(Duration.zero);
        reactiveCounter.value = 30;
        ;
      });

      test('get and set state', () async {
        expectLater(reactiveCounter, emitsInOrder([10, 11, 12]));

        reactiveCounter.value++;
        await Future.delayed(Duration.zero);
        reactiveCounter.value++;
        ;
      });
    });
  });

  group('listen', () {});

  group('close', () {});
}
